package org.databinding.websocket;

import java.io.IOException;
import java.util.HashMap;

import javax.servlet.http.HttpSession;

import org.databinding.Binding;
import org.databinding.DataBindingContext;
import org.databinding.pojo.FormFieldTest;
import org.databinding.pojo.PrimitivePerson;
import org.eclipse.jetty.util.ajax.JSON;
import org.eclipse.jetty.util.ajax.JSONObjectConvertor;
import org.eclipse.jetty.websocket.WebSocket;

public class Socket implements WebSocket {

	private Outbound outbound;
	private Servlet servlet;
	private HttpSession session;
	private DataBindingContext dbc;
	private JSON json;

	public Socket(Servlet servlet, HttpSession session) {
		this.servlet = servlet;
		this.session = session;
		initDataBinding();
	}

	void initDataBinding() {

		json = new JSON();
		json.addConvertor(Message.class, new JSONObjectConvertor());
		json.addConvertor(MessageArray.class, new JSONObjectConvertor());

		if (servlet.getServletContext().getAttribute("DATA_BINDING_CONTEXT") == null)
			servlet.getServletContext().setAttribute("DATA_BINDING_CONTEXT",
					new DataBindingContext(servlet));

		dbc = (DataBindingContext) servlet.getServletContext().getAttribute(
				"DATA_BINDING_CONTEXT");

		// PrimitivePerson reg = new PrimitivePerson();
		// reg.setFirstName("Jack");
		// reg.setLastName("Meyer");
		// reg.setMarried(true);
		// reg.setAge(23);
		//
		// dbc.addBinding(new Binding(reg,"firstName", "firstNameInput"));
		// dbc.addBinding(new Binding(reg,"lastName", "lastNameInput"));
		// dbc.addBinding(new Binding(reg,"married", "marriedInput"));
		// dbc.addBinding(new Binding(reg,"age", "ageInput"));

		FormFieldTest fields = new FormFieldTest();
		fields.setText("text field");
		fields.setPassword("password field");
		fields.setCheckbox(true);
		fields.setRadio("Spring");
		fields.setTextarea("textarea field");
		fields.setSelect("Spring");

		dbc.addBinding(new Binding(fields, "text", "textInput"));
		dbc.addBinding(new Binding(fields, "password", "passwordInput"));
		dbc.addBinding(new Binding(fields, "checkbox", "checkboxInput"));
		dbc.addBinding(new Binding(fields, "radio", "radioInput"));
		dbc.addBinding(new Binding(fields, "textarea", "textareaInput"));
		dbc.addBinding(new Binding(fields, "select", "selectInput"));

//		IcweRegistration conf = new IcweRegistration();
//		conf.setTitle("SAC 2012");
//		conf.setRegistrationType("Member");
//		conf.setConferenceDinner(true);
//		
//		dbc.addBinding(new Binding(conf, "title", "nameId"));
//		dbc.addBinding(new Binding(conf, "registrationType", "registrationTypeId"));
//		dbc.addBinding(new Binding(conf, "conferenceDinner", "conferenceDinnerId"));
		
	}

	public void onConnect(Outbound outbound) {

		System.out.println("CONNECTED");

		this.outbound = outbound;
		servlet.addMember(this);

		dbc.initializeUI();

	}

	public void sendMsg(String msg) {
		try {
			this.outbound.sendMessage(msg);
			System.out.println("SEND: " + msg);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void onMessage(byte frame, byte[] data, int offset, int length) {
	}

	public void onMessage(byte frame, String data) {

		System.out.println("RECEIVED: " + data);

		// a message array was received
		if (data.startsWith("[{")) {

			Object[] obj = (Object[]) json.fromJSON(data);
			for (int i = 0; i < obj.length; i++) {
				HashMap map = (HashMap) obj[i];
				Message msg = new Message((String) map.get("id"),
						(String) map.get("value"));
				MessageHandler msgHandler = new MessageHandler(this, msg);
				dbc.updateModel(msgHandler);
			}

		} else {
			// a single message was received
			HashMap map = (HashMap) json.fromJSON(data);
			Message msg = new Message((String) map.get("id"),
					(String) map.get("value"));
			MessageHandler msgHandler = new MessageHandler(this, msg);
			dbc.updateModel(msgHandler);
		}
	}

	public void onDisconnect() {
		servlet.removeMember(this);
	}

	public void onFragment(boolean more, byte opcode, byte[] data, int offset,
			int length) {
	}
}